package com.stereodustparticles.mooler_caster_console;

import java.awt.Color;

// SDP Mooler Caster Console
// Ridiculously simple radio DJ program, with a 16-slot soundboard and 2-deck music player
// Written by Ben A. (IfYouLikeGoodIdeas)

// This class defines a "spot", consisting of an audio clip and a button to trigger it

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;

public class Spot implements ActionListener {
	private JButton button;
	private JLabel counter;
	private Clip clip = null;
	private Timer timer;
	private LineListener timerCtrl;
	
	// Constants
	private static final LineBorder PLAYING_BORDER = new LineBorder(Color.GREEN, 2);
	
	// Return this spot's button object
	public JButton getButton() {
		return button;
	}
	
	// Return the label object for this spot's counter
	public JLabel getCounter() {
		return counter;
	}

	// Constructor takes in the index number of the new spot object, so that it can be positioned correctly
	public Spot(int i) {
		// Create button and counter objects
		button = new JButton("-"); // TODO recall previous clip
		counter = new JLabel("0:00.0");
		
		// Create the counter's timer
		timer = new Timer(100, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if ( clip != null ) {
					updateCounter(clip.getMicrosecondLength() - clip.getMicrosecondPosition());
				}
			}
		});
		// Create the timer's event handler (that actually also controls the button border...)
		timerCtrl = new LineListener() {
			public void update(LineEvent e) {
				if ( e.getType() == LineEvent.Type.START ) {
					button.setBorder(PLAYING_BORDER);
					timer.start();
				}
				else if ( e.getType() == LineEvent.Type.STOP ) {
					button.setBorder(UIManager.getBorder("Button.border"));
					timer.stop();
					updateCounter(clip.getMicrosecondLength());
				}
			}
		};
		
		// Determine the X/Y position of the new spot button based on the index parameter
		int x = 0;
		switch (i % 4) {
			case 0:
				x = 10;
				break;
			case 1:
				x = 108;
				break;
			case 2:
				x = 207;
				break;
			case 3:
				x = 306;
				break;
		}
		int y = 0;
		switch (i / 4) {
			case 0:
				y = 11;
				break;
			case 1:
				y = 111;
				break;
			case 2:
				y = 211;
				break;
			case 3:
				y = 311;
				break;
		}
		
		// Set the position/size of the button and counter
		button.setBounds(x, y, 89, 89);
		counter.setBounds(x + 36, y + 73, 50, 14);
		
		// Set counter's text alignment and visibility
		counter.setHorizontalAlignment(JLabel.TRAILING);
		counter.setVisible(false);
		
		// Disable the margins on the button so as much text as possible can be displayed
		button.setMargin(new Insets(0, 0, 0, 0));
		
		// Set the button's event handler (to this object, since Spot implements ActionListener...)
		// Hey, at least this particular hack seems to be widely known and accepted! [football helmet]
		button.addActionListener(this);
	}
	
	// Event handler function for the button
	public void actionPerformed(ActionEvent arg0) {
		// If we're not in spot loading, run the Play method (below)
		if ( ! SDPConsole.spotLoading ) {
			play();
		}
		
		// Otherwise, start the loading process
		else {
			// Take the GUI out of the loading state
			SDPConsole.endSpotLoading();
			
			// Start the spot loader
			SpotLoader loader = new SpotLoader(this);
			loader.setVisible(true);
		}
	}
	
	// Load a new clip (from the specified File object) into this Spot
	public void load(URL url, String name) throws LineUnavailableException, IOException, UnsupportedAudioFileException, IllegalArgumentException {
		// If there's a previous clip in the spot, unload it (remove the timer event handler and close it)
		if ( clip != null ) {
			clip.removeLineListener(timerCtrl);
			clip.close();
		}
		
		// Load the file and set up decoding
		// (From http://www.javalobby.org/java/forums/t18465.html)
		AudioInputStream in = AudioSystem.getAudioInputStream(url);
		AudioFormat baseFormat = in.getFormat();
		AudioFormat decodedFormat = new AudioFormat(
			AudioFormat.Encoding.PCM_SIGNED, // Encoding to use
			baseFormat.getSampleRate(),	  // sample rate (same as base format)
			16,				  // sample size in bits (thx to Javazoom)
			baseFormat.getChannels(),	  // # of Channels
			baseFormat.getChannels()*2,	  // Frame Size
			baseFormat.getSampleRate(),	  // Frame Rate
			false				  // Is Big Endian? (No)
		);
		AudioInputStream din = AudioSystem.getAudioInputStream(decodedFormat, in);
		
		// Set up output and decode/load the file into memory
        DataLine.Info info = new DataLine.Info(Clip.class, decodedFormat);
        clip = (Clip)AudioSystem.getLine(info);
        clip.open(din);
        
        // Register the event handler (to start/stop the timer)
        clip.addLineListener(timerCtrl);
        
        // Set button text
        // Render as HTML/CSS for better text formatting
        button.setText("<html><body style='width: 72px;'>" + name + "</body></html>");
        
        // Set the counter and show it
        updateCounter(clip.getMicrosecondLength());
        counter.setVisible(true);
	}
	
	// Play/stop the clip
	public void play() {
		// If there is actually a clip loaded...
		if ( clip != null ) {
			// If it's playing, stop it
			if ( clip.isActive() ) {
				clip.stop();
			}
			// If it's not playing, cue it to the beginning and play it
			else {
				clip.setFramePosition(0);
				clip.start();
			}
		}
	}
	
	// Update the counter
	public void updateCounter(long micro) {
		int tenths = (int)((micro / 100000) % 10);
		int sec = (int)((micro / 1000000) % 60);
		int min = (int)((micro / 1000000) / 60);
		counter.setText(min + ":" + String.format("%02d", sec) + "." + tenths);
	}
}
